@@ -36,8 +36,12 @@
 # define setlocale(Category, Locale)
 #endif
 #include "gettext.h"
-#define _(str) gettext (str)
-#define N_(str) gettext_noop (str)
+
+/*
+ *  _() and N_() strings are both extracted into the .pot file.
+ */
+#define _(str)  gettext (str)
+#define N_(str) gettext_maybe (str)
 
 #ifndef NUL
 #  define NUL '\0'
@@ -278,6 +282,20 @@ static int print_text_dom_dir;
 /* The number used to make the intermediate files unique.  */
 static int sharpid;
 
+/* scribble space. */
+static size_t scribble_size = 1024 + (BUFSIZ * 2);
+static char* scribble;
+
+static int translate_script = 0;
+
+static char*
+gettext_maybe( char* pz )
+{
+  if (translate_script)
+    return gettext (pz);
+  return pz;
+}
+
 #if DEBUG
 # define DEBUG_PRINT(Format, Value) \
     if (debugging_mode)					\
@@ -302,6 +320,50 @@ static void usage __P ((int))
 /* Define a type just for easing ansi2knr's life.  */
 typedef int (*walker_t) __P ((const char *, const char *));
 
+static void
+echo_status( const char*  test_pz,
+	     const char*  ok_format_pz,
+	     const char*  bad_format_pz,
+	     const char*  what_pz,
+	     int die_on_failure )
+{
+  static const char test_okay_z[] = "if %s\nthen ${echo} '";
+  static const char test_else_z[] = "'\nelse ${echo} '";
+  static const char test_fail_z[] = "if %s\nthen : ; else ${echo} '";
+  static const char test_exit_z[] = "'\n  exit 1\nfi\n";
+  static const char test_fi_z[]   = "'\nfi\n";
+
+  if (ok_format_pz != NULL)
+    {
+      fprintf (output, test_okay_z, test_pz);
+      fprintf (output, ok_format_pz, what_pz);
+
+      if (bad_format_pz != NULL)
+        {
+          fputs (test_else_z, output);
+          fprintf (output, bad_format_pz, what_pz);
+        }
+    }
+  else
+    {
+      fprintf (output, test_fail_z, test_pz);
+      fprintf (output, bad_format_pz, what_pz);
+    }
+
+  if (die_on_failure)
+    fputs (test_exit_z, output);
+  else
+    fputs (test_fi_z, output);
+}
+
+static void
+echo_text( const char* format_pz, const char* arg_pz, int cascade )
+{
+  fputs ("  ${echo} 'x -", output);
+  fprintf (output, format_pz, arg_pz);
+  fputs (cascade ? "' &&\n" : "'\n", output);
+}
+
 #if !NO_WALKTREE
 
 /*--------------------------------------------------------------------------.
@@ -362,6 +424,17 @@ walkdown (routine, local_name, restore_name)
   restore_name_copy[ restore_name_length-1 ] = '/';
   restore_name_copy[ restore_name_length   ] = NUL;
 
+  {
+    size_t sz = (sizeof_local_name < sizeof_restore_name)
+      ? sizeof_restore_name : sizeof_local_name;
+    sz = 1024 + (sz * 2);
+    if (scribble_size < sz)
+      {
+        scribble_size = (sz + 4096) & 0x0FFF;
+        scribble = xrealloc (scribble, scribble_size);
+      }
+  }
+
   if ((restore_name_copy[0] == '.') && (restore_name_copy[1] == '/'))
     restore_offset = 2;
   else
@@ -555,9 +628,7 @@ Note: not verifying md5sums.  Consider installing GNU coreutils."));
   if (!quiet_unshar_mode)
     {
       if (vanilla_operation_mode)
-	fputs ("\
-shar_tty= shar_n= shar_c='\n\
-'\n",
+	fputs ("shar_tty= shar_n= shar_c='\n'\n",
 	       output);
       else
 	{
@@ -565,30 +636,13 @@ shar_tty= shar_n= shar_c='\n\
 	    /* Check if /dev/tty exists.  If yes, define shar_tty to
 	       `/dev/tty', else, leave it empty.  */
 
-	    fputs ("\
-if test -n \"`ls /dev/tty 2>/dev/null`\"; then\n\
-  shar_tty=/dev/tty\n\
-else\n\
-  shar_tty=\n\
-fi\n",
-		   output);
+	    fputs (dev_tty_check_z, output);
 
 	  /* Try to find a way to echo a message without newline.  Set
 	     shar_n to `-n' or nothing for an echo option, and shar_c
 	     to `\c' or nothing for a string terminator.  */
 
-	  fputs ("\
-if (echo \"testing\\c\"; echo 1,2,3) | grep c >/dev/null; then\n\
-  if (echo -n testing; echo 1,2,3) | sed s/-n/xn/ | grep xn >/dev/null; then\n\
-    shar_n= shar_c='\n\
-'\n\
-  else\n\
-    shar_n=-n shar_c=\n\
-  fi\n\
-else\n\
-  shar_n= shar_c='\\c'\n\
-fi\n",
-		 output);
+	  fputs (echo_checks_z, output);
 	}
     }
 
@@ -602,22 +656,32 @@ installing GNU `touch'\\'', distributed in GNU File Utilities..."));
 
   if (file_size_limit == 0 || part_number == 1)
     {
+      echo_status ("test ! -d ${lock_dir}", NULL,
+                   N_("lock directory '${lock_dir}' exists"), NULL, 1);
+
       /* Create locking directory.  */
       if (vanilla_operation_mode)
-	fprintf (output, "\
-mkdir _sh%05d || ( echo '%s' '_sh%05d'; exit 1 )\n",
-		 sharpid, N_("failed to create locking directory"), sharpid);
+	echo_status ("mkdir ${lock_dir}", NULL,
+		     N_("failed to create lock directory"), NULL, 1);
       else
-	fprintf (output, "\
-$echo $shar_n 'x -' '%s' \"\\`_sh%05d': \"$shar_c\n\
-if mkdir _sh%05d; then\n\
-  $echo '%s'\n\
-else\n\
-  $echo '%s'\n\
-  exit 1\n\
-fi\n",
-		 N_("lock directory"), sharpid, sharpid, N_("created"),
-		 N_("failed to create"));
+	{
+	  const char* did_pz =
+	    N_("x - created lock directory `'%s\\''.");
+	  const char* not_pz =
+	    N_("x - failed to create lock directory `'%s\\''.");
+	  echo_status ("mkdir ${lock_dir}", did_pz, not_pz, "${lock_dir}", 1);
+	}
+    }
+
+  if (query_user_mode)
+    {
+      fprintf (output, query_answers_z,
+               N_("yes"),  N_("overwrite this file"),
+               N_("no"),   N_("skip this file"),
+               N_("all"),  N_("overwrite all files"),
+               N_("none"), N_("overwrite no files"),
+               N_("help"), N_("explain choices"),
+               N_("quit"), N_("exit immediately"));
     }
 }
 
@@ -676,12 +740,16 @@ generate_mkdir (path)
 
   fprintf (output, "if test ! -d '%s'; then\n", path);
   if (!quiet_unshar_mode)
-    fprintf (output, "\
-  $echo $echo_n 'x -' '%s: '$echo_c\n\
-  if mkdir '%s'; then $echo '%s'; else $echo '%s'; fi\n",
-             path, path, N_("created"), N_("failed to create"));
+    {
+      const char* did_pz =
+	N_("x - created directory `%s'\\''.");
+      const char* not_pz =
+	N_("x - failed to create directory `%s'\\''.");
+      fprintf (output, "  mkdir '%s'\n", path);
+      echo_status ("test $? -eq 0", did_pz, not_pz, path, 1);
+    }
   else
-    fprintf (output, "  mkdir '%s'\n", path);
+    fprintf (output, "  mkdir '%s' || exit 1\n", path);
   fputs ("fi\n", output);
 }
 
@@ -817,67 +885,46 @@ generate_full_header (argc, argv)
 
   if (cut_mark_mode)
     fputs (cut_mark_line, output);
-  fputs ("\
-#!/bin/sh\n",
-	 output);
-  if (archive_name)
-    fprintf (output, "\
-# This is %s, a shell archive (produced by GNU %s %s)\n",
-	     archive_name, PACKAGE, VERSION);
-  else
-    fprintf (output, "\
-# This is a shell archive (produced by GNU %s %s).\n",
-	     PACKAGE, VERSION);
-  fputs ("\
-# To extract the files from this archive, save it to some FILE, remove\n\
-# everything before the `!/bin/sh' line above, then type `sh FILE'.\n\
-#\n",
-	 output);
+  {
+    char* pz = archive_name ? archive_name : "";
+    char* ch = archive_name ? ", a shell" : "a shell";
+
+    fprintf (output, file_leader_z, pz, ch, PACKAGE, VERSION, sharpid);
+  }
 
   time (&now);
   local_time = localtime (&now);
   strftime (buffer, 79, "%Y-%m-%d %H:%M %Z", local_time);
-  fprintf (output, "\
-# Made on %s by <%s>.\n",
+  fprintf (output, "# Made on %s by <%s>.\n",
 	   buffer, submitter_address);
 
   current_directory = xgetcwd ();
   if (current_directory)
     {
-      fprintf (output, "\
-# Source directory was `%s'.\n",
+      fprintf (output, "# Source directory was `%s'.\n",
 	       current_directory);
       free (current_directory);
     }
   else
     error (0, errno, _("Cannot get current directory name"));
 
+  fputs ("#\n# Existing files ", output);
   if (check_existing_mode)
-    fputs ("\
-#\n\
-# Existing files will *not* be overwritten unless `-c' is specified.\n",
+    fputs ("will *not* be overwritten, unless `-c' is specified.\n",
 	   output);
   else if (query_user_mode)
-    fputs ("\
-#\n\
-# existing files MAY be overwritten\n",
-	   output);
+    fputs ("MAY be overwritten.\n", output);
   else
-    fputs ("\
-#\n\
-# existing files WILL be overwritten\n",
-	   output);
+    fputs ("WILL be overwritten.\n", output);
 
   if (query_user_mode)
-    fputs ("\
-# The unsharer will be INTERACTIVELY queried.\n",
+    fputs ("# The unsharer will be INTERACTIVELY queried.\n",
 	   output);
 
   if (vanilla_operation_mode)
     {
-      fputs ("\
-# This format requires very little intelligence at unshar time.\n\
-# ",
+      fputs (
+        "# This format requires very little intelligence at unshar time.\n# ",
 	     output);
       if (check_existing_mode || split_file_mode)
 	fputs ("\"if test\", ", output);
@@ -896,12 +943,7 @@ generate_full_header (argc, argv)
       fprintf (output, "%-75s\n%-75s\n", "#", "#");
     }
 
-  fputs ("\
-#\n\
-# This shar contains:\n\
-# length mode       name\n\
-# ------ ---------- ------------------------------------------\n",
-	 output);
+  fputs (contents_z, output);
 
   for (counter = 0; counter < argc; counter++)
     {
@@ -926,22 +968,72 @@ generate_full_header (argc, argv)
 
   if (split_file_mode)
     {
-
       /* Now check the sequence.  */
-
-      fprintf (output, "\
-if test -r _sh%05d/seq; then\n\
-  $echo '%s'\n\
-  $echo '%s' '`cat _sh%05d/seq`' '%s'\n\
-  exit 1\n\
-fi\n",
-	       sharpid,
-	       N_("Must unpack archives in sequence!"),
-	       N_("Please unpack part"), sharpid, N_("next!")
-);
+      echo_status ("test ! -r ${lock_dir}/seq", NULL,
+                   N_("Archives must be unpacked in sequence!\n"
+                      "Please unpack part '`cat ${lock_dir}/seq`' next."),
+                   NULL, 1);
     }
 }
 
+void
+change_files (const char *restore_name, off_t remaining_size)
+{
+  /* Change to another file.  */
+
+  DEBUG_PRINT (_("New file, remaining %s, "), remaining_size);
+  DEBUG_PRINT (_("Limit still %s\n"), file_size_limit);
+
+  /* Close the "&&" and report an error if any of the above
+     failed.  */
+
+  fputs (" :\n", output);
+  echo_status ("test $? -ne 0", N_("restore of %s failed"), NULL,
+	       restore_name, 0);
+
+  fputs ("'\n${echo} '", output);
+  fprintf (output, N_("End of part %d, continue with part %d"),
+	   part_number, part_number+1);
+  fputs ("'\nexit 0\n", output);
+
+  close_output ();
+
+  /* Clear mkdir_already in case the user unshars out of order.  */
+
+  clear_mkdir_already ();
+
+  /* Form the next filename.  */
+
+  open_output ();
+  if (!quiet_mode)
+    fprintf (stderr, _("Starting file %s\n"), output_filename);
+
+  if (net_headers_mode)
+    {
+      fprintf (output, "Submitted-by: %s\n", submitter_address);
+      fprintf (output, "Archive-name: %s%s%02d\n\n", archive_name,
+	       strchr (archive_name, '/') ? "" : "/part",
+	       part_number ? part_number : 1);
+    }
+
+  if (cut_mark_mode)
+    fputs (cut_mark_line, output);
+
+  {
+    static const char part_z[] =
+      "part %02d of %s ";
+    char *nm = archive_name ? archive_name : "a multipart";
+    char *pz = xmalloc (sizeof(part_z) + strlen(nm) + 16);
+    sprintf (pz, part_z, part_number, nm);
+    fprintf (output, file_leader_z, pz, "", sharpid);
+    free (pz);
+  }
+
+  generate_configure ();
+
+  first_file_position = ftello (output);
+}
+
 /* Prepare a shar script.  */
 
 /*---.
@@ -987,70 +1079,12 @@ shar (local_name, restore_name)
 	       ? struct_stat.st_size + struct_stat.st_size / 3
 	       : struct_stat.st_size)
 	      > remaining_size))
-	{
-
-	  /* Change to another file.  */
-
-	  DEBUG_PRINT (_("New file, remaining %s, "), remaining_size);
-	  DEBUG_PRINT (_("Limit still %s\n"), file_size_limit);
-
-	  /* Close the "&&" and report an error if any of the above
-	     failed.  */
-
-	  /* The following code is somewhat unreadable because marking
-	     the strings is necessary.  The complete string is (nice
-	     formatted:
-: || $echo 'restore of %s failed'\n\
-echo 'End of part %d, continue with part %d'\n\
-exit 0\n",
-           */
-	  fprintf (output, "\
-: || $echo '%s' '%s' '%s'\n\
-$echo '%s' '%d,' '%s' '%d'\n\
-exit 0\n",
-		   N_("restore of"), restore_name, N_("failed"),
-		   N_("End of part"), part_number,
-		   N_("continue with part"), part_number + 1);
-
-	  close_output ();
-
-	  /* Clear mkdir_already in case the user unshars out of order.  */
-
-	  clear_mkdir_already ();
-
-	  /* Form the next filename.  */
-
-	  open_output ();
-	  if (!quiet_mode)
-	    fprintf (stderr, _("Starting file %s\n"), output_filename);
-
-	  if (net_headers_mode)
-	    {
-	      fprintf (output, "Submitted-by: %s\n", submitter_address);
-	      fprintf (output, "Archive-name: %s%s%02d\n\n", archive_name,
-		       strchr (archive_name, '/') ? "" : "/part",
-		       part_number ? part_number : 1);
-	    }
-
-	  if (cut_mark_mode)
-	    fputs (cut_mark_line, output);
-
-	  fprintf (output, "\
-#!/bin/sh\n\
-# This is part %02d of %s.\n",
-		   part_number,
-		   archive_name ? archive_name : "a multipart archive");
-
-	  generate_configure ();
-
-	  first_file_position = ftello (output);
-	}
+        change_files (restore_name, remaining_size);
     }
   else
     remaining_size = 0;		/* give some value to the variable */
 
-  fprintf (output, "\
-# ============= %s ==============\n",
+  fprintf (output, "# ============= %s ==============\n",
 	   restore_name);
 
   generate_mkdir_script (restore_name);
@@ -1214,83 +1248,54 @@ exit 0\n",
 
   if (check_existing_mode)
     {
-      fprintf (output, "\
-if test -f '%s' && test \"$first_param\" != -c; then\n",
+      fprintf (output, "if test -f '%s' && test \"$first_param\" != -c; then\n",
 	       restore_name);
 
       if (query_user_mode)
 	{
-	  fprintf (output, "\
-  case $shar_wish in\n\
-    A*|a*)\n\
-      $echo 'x -' %s '%s' ;;\n\
-    *)\n\
-      $echo $shar_n '? -' %s '%s' '%s' $shar_c\n\
-      if test -n \"$shar_tty\"; then\n\
-	read shar_wish < $shar_tty\n\
-      else\n\
-	read shar_wish\n\
-      fi ;;\n\
-  esac\n\
-  case $shar_wish in\n\
-    Q*|q*)\n\
-      $echo '%s'; exit 1 ;;\n\
-    A*|a*|Y*|y*)\n\
-      shar_skip=no ;;\n\
-    *)\n\
-      shar_skip=yes ;;\n\
-  esac\n\
-else\n\
-  shar_skip=no\n\
-fi\n\
-if test $shar_skip = yes; then\n\
-  $echo 'x -' %s '%s'\n",
-		   N_("overwriting"), restore_name,
-		   N_("overwrite"), restore_name,
-		   N_("[no, yes, all, quit] (no)?"),
-		   N_("extraction aborted"),
-		   N_("SKIPPING"), restore_name);
+          char* pzOverwriting = scribble;
+          char* pzOverwrite;
+
+          sprintf (pzOverwriting, N_("overwriting %s"), restore_name);
+          pzOverwrite = scribble + strlen(pzOverwriting) + 2;
+          sprintf (pzOverwrite, N_("overwrite %s"), restore_name);
+
+          fprintf (output, query_user_z, pzOverwriting, pzOverwrite);
+
+          sprintf (pzOverwriting, N_("SKIPPING %s"), restore_name);
+          fprintf (output, query_check_z, N_("extraction aborted"),
+		   pzOverwriting, pzOverwriting);
 	}
       else
-	fprintf (output, "\
-  $echo 'x -' %s '%s' '%s'\n",
-		 N_("SKIPPING"), restore_name, N_("(file already exists)"));
+        echo_text (N_("SKIPPING %s (file already exists)"),
+                   restore_name, 0);
 
       if (split_file_mode)
-	fprintf (output, "\
-  rm -f _sh%05d/new\n",
-		 sharpid);
-
-      fputs ("\
-else\n",
-	     output);
-
-      if (split_file_mode)
-	fprintf (output, "\
-  > _sh%05d/new\n",
-		 sharpid);
+	fputs ("  rm -f ${lock_dir}/new\nelse\n  > ${lock_dir}/new\n", output);
+      else
+	fputs ("else\n", output);
     }
 
   if (!quiet_mode)
     error (0, 0, _("Saving %s (%s)"), local_name, file_type);
 
   if (!quiet_unshar_mode)
-    fprintf (output, "\
-  $echo 'x -' %s '%s' '%s'\n",
-	     N_("extracting"), restore_name, file_type_remote);
+    {
+      sprintf (scribble, N_("x - extracting %s %s"),
+               restore_name, file_type_remote);
+      fprintf (output, echo_string_z, scribble);
+    }
 
   if (struct_stat.st_size == 0)
     {
 
       /* Just touch the file, or empty it if it exists.  */
 
-      fprintf (output, "\
-  > '%s' &&\n",
+      fprintf (output, "  > '%s' &&\n",
 	       restore_name);
     }
   else
     {
-
       /* Run sed for non-empty files.  */
 
       if (uuencoded_file_mode)
@@ -1298,11 +1303,10 @@ else\n",
 
 	  /* Run sed through uudecode (via temp file if might get split).  */
 
-	  fprintf (output, "\
-  sed 's/^%c//' << '%s' ",
+	  fprintf (output, "  sed 's/^%c//' << '%s' ",
 		   line_prefix, here_delimiter);
 	  if (inhibit_piping_mode)
-	    fprintf (output, "> _sh%05d/uue &&\n", sharpid);
+	    fprintf (output, "> ${lock_dir}/uue &&\n");
 	  else
 	    fputs ("| uudecode &&\n", output);
 	}
@@ -1311,8 +1315,7 @@ else\n",
 
 	  /* Just run it into the file.  */
 
-	  fprintf (output, "\
-  sed 's/^%c//' << '%s' > '%s' &&\n",
+	  fprintf (output, "  sed 's/^%c//' << '%s' > '%s' &&\n",
 		   line_prefix, here_delimiter, restore_name);
 	}
 
@@ -1393,35 +1396,34 @@ else\n",
 	      /* Close the "&&" and report an error if any of the above
 		 failed.  */
 
-	      fprintf (output, "\
-  : || $echo '%s' '%s' '%s'\n",
-		       N_("restore of"), restore_name, N_("failed"));
+	      fputs (" :\n", output);
+	      echo_status ("test $? -ne 0", N_("restore of %s failed\n"), NULL,
+			   restore_name, 0);
 
 	      if (check_existing_mode)
-		fputs ("\
-fi\n",
-		       output);
+		fputs ("fi\n", output);
 
 	      if (quiet_unshar_mode)
-		fprintf (output, "\
-$echo '%s' '%d,' '%s' '%d'\n",
-			 N_("End of part"), part_number,
-			 N_("continue with part"), part_number + 1);
+                {
+                  sprintf (scribble,
+                           N_("End of part %ld, continue with part %ld"),
+			   (long)part_number, (long)part_number + 1);
+                  fprintf (output, echo_string_z, scribble);
+                }
 	      else
-		fprintf (output, "\
-$echo '%s' '%s' '%s' '%d'\n\
-$echo '%s' '%s' '%s' '%d'\n",
-			 N_("End of"),
-			 archive_name ? archive_name : N_("archive"),
-			 N_("part"),
-			 part_number,
-			 N_("File"), restore_name,
-			 N_("is continued in part"), part_number + 1);
+		{
+                  sprintf (scribble, N_("End of %s part %d"),
+                           archive_name ? archive_name : N_("archive"),
+                           part_number);
+                  fprintf (output, echo_string_z, scribble);
 
-	      fprintf (output, "\
-echo %d > _sh%05d/seq\n\
-exit 0\n",
-		       part_number + 1, sharpid);
+                  sprintf (scribble, N_("File %s is continued in part %d"),
+                           restore_name, (long)part_number + 1);
+                  fprintf (output, echo_string_z, scribble);
+		}
+
+	      fprintf (output, "echo %d > ${lock_dir}/seq\nexit 0\n",
+		       part_number + 1);
 
 	      if (part_number == 1)
 		{
@@ -1430,11 +1432,9 @@ exit 0\n",
 
 		  fseeko (output, archive_type_position, SEEK_SET);
 		  fprintf (output, "%-75s\n%-75s\n",
-			   "\
-# This is part 1 of a multipart archive.",
-			   "\
-# Do not concatenate these parts, unpack them in order with `/bin/sh'.");
-			   }
+		   "# This is part 1 of a multipart archive.",
+		   "# Do not concatenate these parts, unpack them in order with `/bin/sh'.");
+                }
 	      close_output ();
 
 	      /* Next! */
@@ -1453,61 +1453,42 @@ exit 0\n",
 	      if (cut_mark_mode)
 		fputs (cut_mark_line, output);
 
-	      fprintf (output, "\
-#!/bin/sh\n\
-# This is `%s' (part %d of %s).\n\
-# Do not concatenate these parts, unpack them in order with `/bin/sh'.\n\
-# File `%s' is being continued...\n\
-#\n",
+	      fprintf (output, continue_archive_z,
 		       basename (output_filename), part_number,
 		       archive_name ? archive_name : "a multipart archive",
-		       restore_name);
+		       restore_name, sharpid);
 
 	      generate_configure ();
 
-	      fprintf (output, "\
-if test ! -r _sh%05d/seq; then\n\
-  $echo '%s'\n\
-  exit 1\n\
-fi\n\
-shar_sequence=`cat _sh%05d/seq`\n\
-if test \"$shar_sequence\" != %d; then\n\
-  $echo '%s' \"$shar_sequence\" '%s'\n\
-  exit 1\n\
-fi\n",
-		       sharpid,
+	      fprintf (output, seq_check_z,
 		       N_("Please unpack part 1 first!"),
-		       sharpid,
-		       part_number,
-		       N_("Please unpack part"),
-		       N_("next!"));
+		       part_number);
+
+              echo_status ("test $? -eq 0", NULL,
+                           N_("Please unpack part '${shar_sequence}' next!"),
+                           NULL, 1);
 
 	      if (check_existing_mode)
 		{
 		  if (quiet_unshar_mode)
-		    fprintf (output, "\
-if test -f _sh%05d/new; then\n",
-			     sharpid);
+		    fprintf (output, "if test -f ${lock_dir}/new; then\n");
 		  else
-		    fprintf (output, "\
-if test ! -f _sh%05d/new; then\n\
-  $echo 'x -' '%s' '%s'\n\
-else\n",
-			     sharpid,
-			     N_("STILL SKIPPING"), restore_name);
+                    {
+                      fputs ("if test ! -f ${lock_dir}/new\nthen echo 'x - ",
+                             output);
+                      fprintf (output, N_("STILL SKIPPING %s"), restore_name);
+                      fputs ("'\nelse\n", output);
+                    }
 		}
 
 	      if (!quiet_mode)
 		fprintf (stderr, _("Starting file %s\n"), output_filename);
 	      if (!quiet_unshar_mode)
-		fprintf (output, "\
-  $echo 'x -' '%s' '%s'\n",
-			 N_("continuing file"), restore_name);
-	      fprintf (output, "\
-  sed 's/^%c//' << '%s' >> ",
+                echo_text (N_("continuing file %s"), restore_name, 0);
+	      fprintf (output, "  sed 's/^%c//' << '%s' >> ",
 		       line_prefix, here_delimiter);
 	      if (uuencoded_file_mode)
-		fprintf (output, "_sh%05d/uue &&\n", sharpid);
+		fprintf (output, "${lock_dir}/uue &&\n");
 	      else
 		fprintf (output, "%s &&\n", restore_name);
 	      remaining_size = file_size_limit;
@@ -1521,22 +1502,16 @@ else\n",
 
       fprintf (output, "%s\n", here_delimiter);
       if (split_flag && !quiet_unshar_mode)
-	fprintf (output, "\
-  $echo '%s' '%s' '%s' &&\n",
-		 N_("File"), restore_name, N_("is complete"));
+        echo_text (N_("File %s is complete"), restore_name, 1);
 
       /* If this file was uuencoded w/Split, decode it and drop the temp.  */
 
       if (uuencoded_file_mode && inhibit_piping_mode)
 	{
 	  if (!quiet_unshar_mode)
-	    fprintf (output, "\
-  $echo '%s' '%s' &&\n",
-		     N_("uudecoding file"), restore_name);
+            echo_text (N_("uudecoding file %s"), restore_name, 1);
 
-	  fprintf (output, "\
-  uudecode _sh%05d/uue < _sh%05d/uue &&\n",
-		   sharpid, sharpid);
+	  fprintf (output, "  uudecode ${lock_dir}/uue < ${lock_dir}/uue &&\n");
 	}
 
       /* If this file was compressed, uncompress it and drop the temp.  */
@@ -1544,35 +1519,26 @@ else\n",
       if (compressed_file_mode)
 	{
 	  if (!quiet_unshar_mode)
-	    fprintf (output, "\
-  $echo '%s' '%s' &&\n",
-		     N_("uncompressing file"), restore_name);
+            echo_text (N_("uncompressing file %s"), restore_name, 1);
 
-	  fprintf (output, "\
-  compress -d < _sh%05d/cmp > '%s' &&\n",
-		   sharpid, restore_name);
+	  fprintf (output, "  compress -d < ${lock_dir}/cmp > '%s' &&\n",
+		   restore_name);
 	}
       else if (gzipped_file_mode)
 	{
 	  if (!quiet_unshar_mode)
-	    fprintf (output, "\
-  $echo '%s' '%s' &&\n",
-		     N_("gunzipping file"), restore_name);
+            echo_text (N_("gunzipping file %s"), restore_name, 1);
 
-	  fprintf (output, "\
-  gzip -d < _sh%05d/gzi > '%s' &&\n",
-		   sharpid, restore_name);
+	  fprintf (output, "  gzip -d < ${lock_dir}/gzi > '%s' &&\n",
+		   restore_name);
 	}
       else if (bzipped_file_mode)
 	{
 	  if (!quiet_unshar_mode)
-	    fprintf (output, "\
-  $echo '%s' '%s' &&\n",
-		     N_("bunzipping file"), restore_name);
+            echo_text (N_("bunzipping file %s"), restore_name, 1);
 
-	  fprintf (output, "\
-  bzip2 -d < _sh%05d/bzi > '%s' &&\n",
-		   sharpid, restore_name);
+	  fprintf (output, "  bzip2 -d < ${lock_dir}/bzi > '%s' &&\n",
+		   restore_name);
 	}
     }
 
@@ -1596,10 +1562,9 @@ else\n",
 
       /* Close the "&&" and report an error if any of the above
 	 failed.  */
-
-      fprintf (output, "\
-  : || $echo '%s' '%s' '%s'\n",
-	       N_("restore of"), restore_name, N_("failed"));
+      fputs (":\n", output);
+      echo_status ("test $? -ne 0", N_("restore of %s failed"), NULL,
+		   restore_name, 0);
     }
   else
     {
@@ -1609,15 +1574,13 @@ else\n",
 
       /* Set the permissions as they were.  */
 
-      fprintf (output, "\
-  chmod %04o '%s' ||\n",
+      fprintf (output, "  chmod %04o '%s'\n",
 	       (unsigned) (struct_stat.st_mode & 0777), restore_name);
 
       /* Report an error if any of the above failed.  */
 
-      fprintf (output, "\
-  $echo '%s' '%s' '%s'\n",
-	       N_("restore of"), restore_name, N_("failed"));
+      echo_status ("test $? -ne 0", N_("restore of %s failed"), NULL,
+		   restore_name, 0);
 
       if (md5_count_mode && (fp = fopen (local_name, "r")) != NULL
 	  && md5_stream (fp, md5buffer) == 0)
@@ -1632,8 +1595,7 @@ else\n",
 	  for (cnt = 0; cnt < 16; ++cnt)
 	    fprintf (output, "%02x", md5buffer[cnt]);
 
-	  fprintf (output, " %c%s\n\
-%s\n",
+	  fprintf (output, " %c%s\n%s\n",
 		   ' ', restore_name, here_delimiter);
 	  /* This  ^^^ space is not necessarily a parameter now.  But it
 	     is a flag for binary/text mode and will perhaps be used later.  */
@@ -1649,16 +1611,14 @@ else\n",
 	  FILE *pfp;
 	  char command[BUFSIZ];
 
-	  sprintf (command, "%s '%s'", CHARACTER_COUNT_COMMAND, local_name);
+	  sprintf (command, "LC_ALL=C wc -c < '%s'", local_name);
 	  if (pfp = popen (command, "r"), pfp)
 	    {
 	      char wc[BUFSIZ];
-	      const char *prefix = "";
 
 	      if (did_md5)
 		{
 		  fputs ("  else\n", output);
-		  prefix = "  ";
 		}
 
               {
@@ -1682,14 +1642,12 @@ else\n",
                 *pz = NUL;
               }
 
-	      fprintf (output, "\
-%s  shar_count=\"`%s '%s'`\"\n\
-%s  test %s -eq \"$shar_count\" ||\n\
-%s  $echo '%s:' '%s' '%s,' '%s' \"$shar_count!\"\n",
-		       prefix, CHARACTER_COUNT_COMMAND, restore_name,
-		       prefix, wc,
-		       prefix, restore_name, N_("original size"), wc,
-		       N_("current size"));
+	      fprintf (output,
+                       "test `LC_ALL=C wc -c < '%s'` -ne %s && \\\n  ${echo} ",
+                       restore_name, wc);
+              fprintf (output,
+                       N_("'restoration warning:  size of %s is not %s'\n"),
+                       restore_name, wc);
 	      pclose (pfp);
 	    }
 	}
@@ -1700,9 +1658,7 @@ else\n",
   /* If the exists option is in place close the if.  */
 
   if (check_existing_mode)
-    fputs ("\
-fi\n",
-	   output);
+    fputs ("fi\n", output);
 
   return 0;
 }
@@ -1815,6 +1771,7 @@ Controlling the shar headers:\n\
   -s, --submitter=ADDRESS   override the submitter name\n\
   -a, --net-headers         output Submitted-by: & Archive-name: headers\n\
   -c, --cut-mark            start the shar with a cut line\n\
+  -t, --translate           translate messages in the script\n\
 \n\
 Selecting how files are stocked:\n\
   -M, --mixed-uuencode         dynamically decide uuencoding (default)\n\
@@ -1848,17 +1805,14 @@ Producing different kinds of shars:\n\
   -f, --basename            restore in one directory, despite hierarchy\n\
       --no-i18n             do not produce internationalized shell script\n"),
 	     stdout);
-      fputs (_("\
-\n\
+      fputs (_("\n\
 Option -o is required with -l or -L, option -n is required with -a.\n"),
 	     stdout);
 #if HAVE_COMPRESS
-      fputs (_("\
-Option -g implies -z, option -b implies -Z.\n"),
+      fputs (_("Option -g implies -z, option -b implies -Z.\n"),
 	     stdout);
 #else
-      fputs (_("\
-Option -g implies -z.\n"),
+      fputs (_("Option -g implies -z.\n"),
 	     stdout);
 #endif
       /* TRANSLATORS: add the contact address for your translation team! */
@@ -1901,6 +1855,7 @@ static const struct option long_options[] =
   {"stdin-file-list", no_argument, NULL, 'S'},
   {"submitter", required_argument, NULL, 's'},
   {"text-files", no_argument, NULL, 'T'},
+  {"translate", no_argument, NULL, 't'},
   {"uuencode", no_argument, NULL, 'B'},
   {"vanilla-operation", no_argument, NULL, 'V'},
   {"whole-size-limit", required_argument, NULL, 'l'},
@@ -2059,6 +2014,10 @@ main (argc, argv)
 	vanilla_operation_mode = 1;
 	break;
 
+      case 't':
+        translate_script = 1;
+        break;
+
       case 'T':
 	set_file_mode ('T');
 	break;
@@ -2176,7 +2135,7 @@ This system doesn't support -Z ('compress'), use -z instead"));
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-	      "1994, 1995, 1996");
+	      "1994, 1995, 1996, 2004, 2005");
       exit (EXIT_SUCCESS);
     }
 
@@ -2313,14 +2272,15 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
       if (net_headers_mode)
 	error (0, 0, _("PLEASE avoid -X shars on Usenet or public networks"));
 
-      fputs ("\
-shar_wish=\n", output);
+      fputs ("shar_wish=\n", output);
     }
 
   first_file_position = ftello (output);
 
   /* Process positional parameters and files.  */
 
+  scribble = xmalloc (scribble_size);
+
   for (; optind < argc; optind++)
     {
       char* arg = argv[optind];
@@ -2350,23 +2310,27 @@ shar_wish=\n", output);
 
   if (split_file_mode && part_number > 1)
     {
-      fprintf (output, "\
-$echo '%s'\n",
+      fprintf (output, "$echo '%s'\n",
 	       N_("You have unpacked the last part"));
       if (quiet_mode)
 	fprintf (stderr, _("Created %d files\n"), part_number);
     }
 
-  fprintf (output, "\
-$echo $shar_n 'x -' '%s' \"\\`_sh%05d': \" $shar_c\n\
-if rm -fr _sh%05d; then\n\
-  $echo '%s'\n\
-else\n\
-  $echo '%s'\n\
-fi\n\
-exit 0\n",
-	   N_("lock directory"), sharpid, sharpid, N_("removed"),
-	   N_("failed to remove"));
+  {
+    const char* did_pz =
+      N_("x - removed lock directory `'%s\\''.");
+    const char* not_pz =
+      N_("x - failed to remove lock directory `'%s\\''.");
 
+    echo_status ("rm -fr ${lock_dir}", did_pz, not_pz, "${lock_dir}", 1);
+  }
+  fputs ("exit 0\n", output);
   exit (status);
 }
+/*
+ * Local Variables:
+ * mode: C
+ * c-file-style: "gnu"
+ * indent-tabs-mode: nil
+ * End:
+ * end of agen5/autogen.c */
