@@ -3,7 +3,7 @@ static const char cright_years_z[] =
    "1994, 1995, 1996, 2002, 2005";
 
 /* uudecode utility.
-   Copyright (C) 1994, 1995, 1996, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1996, 2002, 2005, 2006 Free Software Foundation, Inc.
 
    This product is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -80,6 +80,13 @@ static const char cright_years_z[] =
 #include <pwd.h>
 #include "getopt.h"
 
+#define UU_MODE_BITS(_M)        ((_M) & (S_IRWXU | S_IRWXG | S_IRWXO))
+#if HAVE_FCHMOD
+#define UU_CHMOD(_n, _fd, _m)   fchmod ((_fd), UU_MODE_BITS(_m))
+#else
+#define UU_CHMOD(_n, _fd, _m)   chmod ((_n), UU_MODE_BITS(_m))
+#endif
+
 struct passwd *getpwnam ();
 
 static struct option longopts[] =
@@ -358,36 +365,45 @@ decode (inname, forced_outname)
 	}
     }
 
-  /* Check out file if it exists */
-  if (strcmp (outname, "/dev/stdout") != 0 && strcmp (outname, "-") != 0
-      && !access(outname, F_OK)) {
-    if (lstat(outname, &attr) == -1) {
-      error (0, errno, _("cannot access %s"), outname);
-      return 1;
-    }
-    if (S_ISFIFO(attr.st_mode)){
-      error (0, errno, _("denied writing FIFO (%s)"), outname);
-      return 1;
-    }
-    if (S_ISLNK(attr.st_mode)) {
-      error (0, errno, _("not following symlink (%s)"), outname);
-      return 1;
-    }
-  }
-
   /* Create output file and set mode.  */
 
-  if (strcmp (outname, "/dev/stdout") != 0 && strcmp (outname, "-") != 0
-      && (freopen (outname, FOPEN_WRITE_BINARY, stdout) == NULL
-#if HAVE_FCHMOD
-	  || fchmod (fileno (stdout), mode & (S_IRWXU | S_IRWXG | S_IRWXO))
-#else
-	  || chmod (outname, mode & (S_IRWXU | S_IRWXG | S_IRWXO))
-#endif
-	  ))
+  if (  (strcmp (outname, "/dev/stdout") != 0)
+     && (strcmp (outname, "-") != 0) )
     {
-      error (0, errno, "%s: %s", outname, inname);
-      return 1;
+      FILE* fp;
+
+      /* Check out file if it exists */
+      if (!access(outname, F_OK))
+        {
+          if (lstat(outname, &attr) == -1)
+            {
+              error (0, errno, _("cannot access %s"), outname);
+              return 1;
+            }
+          if (S_ISFIFO(attr.st_mode))
+            {
+              error (0, errno, _("denied writing FIFO (%s)"), outname);
+              return 1;
+            }
+          if (S_ISLNK(attr.st_mode))
+            {
+              error (0, errno, _("not following symlink (%s)"), outname);
+              return 1;
+            }
+        }
+
+      fp = freopen (outname, FOPEN_WRITE_BINARY, stdout);
+      if (fp != stdout)
+        {
+          error (0, errno, _("freopen of %s"), outname);
+          return 1;
+        }
+
+      if (UU_CHMOD(outname, fileno (fp), mode) != 0)
+        {
+          error (0, errno, _("chmod of %s"), outname);
+          return 1;
+        }
     }
 
   /* We differenciate decoding standard UU encoding and base64.  A
@@ -501,3 +517,11 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 
   exit (exit_status);
 }
+
+/*
+ * Local Variables:
+ * mode: C
+ * c-file-style: "gnu"
+ * indent-tabs-mode: nil
+ * End:
+ * end of agen5/autogen.c */
